/////////////////////////////////////////////////////////////////////////////
// tvgames.cpp: application implementation file



#include "stdafx.h"
#include "resource.h"

#include <math.h>	// Needed for sin,cos Functions

#include <bios.h>	// Needed for Keyboard Reading
#include <ctype.h>	// Needed for Keyboard Reading

#include <time.h>	// Needed for RAND Function
#include <stdio.h>	// Needed for RAND Function
#include <stdlib.h>	// Needed for RAND Function


#include "dlg.h"
#include "tvgames.h"


/////////////////////////////////////////////////////////////////////////////
// Global Data Declaration

// COLOR Data Structures
COLORREF COLOR_CODES[] = {  BLACK,
							WHITE,
							DARKGREY,
							GREY,
							BROWN,
							RED,
							YELLOW,
							LIGHTYELLOW,
							GREEN,
							LIGHTGREEN,
							CYAN,
							LIGHTCYAN,
							BLUE,
							LIGHTBLUE,
							MAGENTA,
							FUCSIA };

extern const char* COLOR_NAMES[] = {	"Black",
										"White",
										"DarkGrey",
										"Grey",
										"Brown",
										"Red",
										"Yellow",
										"LightYellow",
										"Green",
										"LightGreen",
										"Cyan",
										"LightCyan",
										"Blue",
										"LightBlue",
										"Magenta",
										"Fucsia" };

extern const int NUM_COLORS = sizeof(COLOR_NAMES)/sizeof(COLOR_NAMES[0]);


/////////////////////////////////////////////////////////////////////////////
// Global Functions

// Color Global Functions
CString GetColorName(COLORREF color)
{
    for (int i=0; i<NUM_COLORS; i++)
    	if (color==COLOR_CODES[i])
    		return COLOR_NAMES[i];
    		
	return "Unknown Color";
}

COLORREF GetColorValue(const char* colorName)
{
	CString color(colorName);

	for (int i=0; i<NUM_COLORS; i++)
		if (color.CompareNoCase(COLOR_NAMES[i])==0)
			return COLOR_CODES[i];

	return BLACK;
}

// Keyboard Recognition Function
CString GetVirtualKeyName(UINT nVKCode)
{
	UINT VK_0 = 0x30;
	UINT VK_9 = 0x39;
	UINT VK_A = 0x41;
	UINT VK_Z = 0x5a;
	
	CString res;
	
	// Digit?
	if ((nVKCode>=VK_0) && (nVKCode<=VK_9))
		res += (char)('0' + (nVKCode - VK_0));
		
	// Character?
	if ((nVKCode>=VK_A) && (nVKCode<=VK_Z))
		res += (char)('a' + (nVKCode - VK_A));
	
	// Numeric Pad Digit?
	if ((nVKCode>=VK_NUMPAD0) && (nVKCode<=VK_NUMPAD9))
	{	
		res = "NPad";
		res += (char)('0' + (nVKCode - VK_NUMPAD0));
	}
	
	// Function Key?
	if ((nVKCode>=VK_F1) && (nVKCode<=VK_F24))
	{	
		res = "F";
		
		char buffer[4];
		res += _itoa((nVKCode - VK_F1 + 1),buffer,10);
	}
	
	switch (nVKCode)
	{
		case VK_BACK		: res = "BkSpc" ; break;
		case VK_TAB			: res = "TAB" ; break;
		case VK_CLEAR		: res = "Clear" ; break;
		case VK_RETURN		: res = "RET" ; break;
		case VK_SHIFT		: res = "Shift" ; break;
		case VK_CONTROL		: res = "Ctrl" ; break;
		case VK_MENU		: res = "Alt" ; break;
		case VK_PAUSE		: res = "Pause" ; break;
		case VK_CAPITAL		: res = "Caps" ; break;
		case VK_ESCAPE		: res = "Esc" ; break;
		case VK_SPACE		: res = "Space" ; break;
		case VK_PRIOR		: res = "PgUp" ; break;
		case VK_NEXT		: res = "PgDw" ; break;
		case VK_END			: res = "End" ; break;
		case VK_HOME		: res = "Home" ; break;
		case VK_LEFT		: res = "Left" ; break;
		case VK_UP			: res = "Up" ; break;
		case VK_RIGHT		: res = "Right" ; break;
		case VK_DOWN		: res = "Down" ; break;
		case VK_SELECT		: res = "Sel" ; break;
		case VK_EXECUTE		: res = "Exe" ; break;
		case VK_SNAPSHOT	: res = "Print" ; break;
		case VK_INSERT		: res = "Ins" ; break;
		case VK_DELETE		: res = "Del" ; break;
		case VK_HELP		: res = "Help" ; break;
		case VK_MULTIPLY	: res = "NPad *" ; break;
		case VK_ADD			: res = "NPad +" ; break;
		case VK_SEPARATOR	: res = "Sep" ; break;
		case VK_SUBTRACT	: res = "NPad -" ; break;
		case VK_DECIMAL		: res = "NPad ." ; break;
		case VK_DIVIDE		: res = "NPad /" ; break;
		case VK_NUMLOCK		: res = "NLock" ; break;
		case VK_SCROLL		: res = "Scroll" ; break;
	}
	
	return res;
}


/////////////////////////////////////////////////////////////////////////////
// CMainWindow constructor:
// Create the window with the appropriate style, size, menu, etc.

CMainWindow::CMainWindow()
{
	// Disable Title Bar
	CString className;       
	HBRUSH hBrush = (HBRUSH)(COLOR_WINDOW + 1);		// System Default
	HICON hIcon = AfxGetApp()->LoadIcon(AFX_IDI_STD_FRAME);
	HCURSOR hCursor = AfxGetApp()->LoadCursor(IDC_NULLCURSOR);

	m_appName.LoadString(AFX_IDS_APP_TITLE);
	className = AfxRegisterWndClass(CS_DBLCLKS|CS_BYTEALIGNWINDOW,
												hCursor,hBrush,hIcon);
	
	Create(className,m_appName,
			WS_VISIBLE|WS_OVERLAPPED|WS_CAPTION|WS_MAXIMIZE|WS_MINIMIZEBOX,
			CFrameWnd::rectDefault,
			NULL,
			"MainMenu");

	// Default Values
	m_paused = FALSE;
	m_started = FALSE;
	m_running = FALSE;
	m_areScoresShown = FALSE;
	
	m_ballXPos = 0;
	m_ballYPos = 0;
	m_ballXPrevPos = 0;
	m_ballYPrevPos = 0;
	m_ballXVel = 0;
	m_ballYVel = 0;
	m_ballEffect = 0;
	m_ballNextEffect = 0;

	m_velP1 = 0;                    
	m_posP1 = 0;
	m_prevPosP1 = 0;
	m_scoreP1 = 0;
	
	m_velP2 = 0;                    
	m_posP2 = 0;
	m_prevPosP2 = 0;
	m_scoreP2 = 0;

	m_computerCount = 0;
}

void CMainWindow::InitMainWindow()
{
	// Set RAND Seed
	srand((unsigned)time(NULL));

	// Load Accelerator Table
	LoadAccelTable("MAINACCELERATOR");

	// Load Window Rect
	GetClientRect(m_winRect);
	
	// Calculate Score Rect
	m_scoresRect.left = (m_winRect.Width() - SCORES_HORZ_SPACING)/2
												- 5*SCORES_CHAR_WIDTH;
	m_scoresRect.right = (m_winRect.Width() + SCORES_HORZ_SPACING)/2
												+ 5*SCORES_CHAR_WIDTH;
	m_scoresRect.top = MARGIN_WIDTH + SCORES_VERT_SPACING;
	m_scoresRect.bottom = m_scoresRect.top + 5*SCORES_CHAR_WIDTH;

	m_batHeight = m_largeBats ? m_bigBatHeight : m_smallBatHeight;

	// Load Game Board
	if (m_currGame.CompareNoCase(PELOTA_STR)==0)
		OnGamePelota();

	if (m_currGame.CompareNoCase(TENNIS_STR)==0)
		OnGameTennis();

	if (m_currGame.CompareNoCase(HOCKEY_STR)==0)
		OnGameHockey();

	if (m_currGame.CompareNoCase(TRAINING_STR)==0)
		OnGameTraining();
}
 
CMainWindow::~CMainWindow()
{
	KillTimer(TV_GAMES_TIMER_ID);                 

	// Delete Ball Bitmap
	m_ballBmp.DeleteObject();
}

 
/////////////////////////////////////////////////////////////////////////////
// Keyboard Management

void CMainWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (nChar==VK_ESCAPE)
	{
		// Hide Window
		if (m_running)
			OnGamePause();
			
		ShowWindow(SW_MINIMIZE);
		return;
	}
	
	// Default Processing for Repeated Keys
	if ((nChar==VK_CONTROL)
		|| !m_started || m_paused || (nFlags & 0x4000))
	{
		CFrameWnd::OnKeyDown(nChar,nRepCnt,nFlags);
		return;
	}
	
	if (m_started && (!m_running))
	{
		m_running = TRUE;
	
		// Hide Scores?
		if ((!m_scoresAlwaysOnTop) && (m_areScoresShown))
			HideScores();
		
		// Play Current Ball
		SetTimer(TV_GAMES_TIMER_ID,TV_GAMES_TIMER_DELAY,NULL);
	}
	
	if (nChar==m_upKeyP1)
	{
		m_velP1 = -BAT_INITIAL_SPEED;
	}
	
	if (nChar==m_downKeyP1)
	{
		m_velP1 = BAT_INITIAL_SPEED;
	}
	
	if ((nChar==m_upKeyP2)
		&& (m_computerSpeed==COMPUTER_NOT_PLAYING))
	{
		m_velP2 = -BAT_INITIAL_SPEED;   	
	}
	
	if ((nChar==m_downKeyP2)
		&& (m_computerSpeed==COMPUTER_NOT_PLAYING))
	{
		m_velP2 = BAT_INITIAL_SPEED;
	}
}

void CMainWindow::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (!m_started || m_paused || (nFlags & 0x2000))
	{
		CFrameWnd::OnKeyUp(nChar,nRepCnt,nFlags);
		return;
	}
	
	if ((nChar==m_upKeyP1)
		&& (m_velP1<0))
	{
		m_velP1 = 0;
	}
	
	if ((nChar==m_downKeyP1)
		&& (m_velP1>0))
	{
		m_velP1 = 0;
	}

	if ((nChar==m_upKeyP2)
		&& (m_velP2<0))
	{
		m_velP2 = 0;
	}
	
	if ((nChar==m_downKeyP2)
		&& (m_velP2>0))
	{
		m_velP2 = 0;
	}
}


/////////////////////////////////////////////////////////////////////////////
// Game Management

void CMainWindow::OnTimer(UINT nIDEvent)
{
	if (nIDEvent==TV_GAMES_TIMER_ID)
	{
		// Store Prev Positions
		m_prevPosP1 = m_posP1;
		m_prevPosP2 = m_posP2; 
		m_ballXPrevPos = m_ballXPos;
		m_ballYPrevPos = m_ballYPos;

		// Move Player 1 Bats
		m_posP1 += m_velP1;  
		m_posP1 = __max(MARGIN_WIDTH,m_posP1);
		m_posP1 = __min(m_posP1,(m_winRect.Height() - MARGIN_WIDTH - m_batHeight));
		
		if (m_accelerated) 
		{
			if ((m_velP1>0) && (m_velP1+BAT_SPEED_RATIO<=BAT_MAX_SPEED))
				m_velP1 += BAT_SPEED_RATIO;
			
			if ((m_velP1<0) && (m_velP1-BAT_SPEED_RATIO>=-BAT_MAX_SPEED))
				m_velP1 -= BAT_SPEED_RATIO;
		}	

		// Computer Playing?
		if (m_computerSpeed!=COMPUTER_NOT_PLAYING)
		{
			m_computerCount++;

			if (m_computerCount>=m_computerSpeed)
			{
				// Move Computer Bat
				m_computerCount = 0;
				
				MoveComputerBat();
			}	
		}
		
		// Move Player 2 Bats
		m_posP2 += m_velP2;
		m_posP2 = __max(MARGIN_WIDTH,m_posP2);
		m_posP2 = __min(m_posP2,(m_winRect.Height() - MARGIN_WIDTH - m_batHeight));

		if (m_accelerated) 
		{
			if ((m_velP2>0) && (m_velP2+BAT_SPEED_RATIO<=BAT_MAX_SPEED))
				m_velP2 += BAT_SPEED_RATIO;

			if ((m_velP2<0) && (m_velP2-BAT_SPEED_RATIO>=-BAT_MAX_SPEED))
				m_velP2 -= BAT_SPEED_RATIO;
		}

		// Move Ball
		CalculateBallPos();
		
		RedrawBall(TRUE);	// Hide Ball
		RedrawRackets();	// Move Rackets
		RedrawBall();		// Draw Ball
	}
	else
		CFrameWnd ::OnTimer(nIDEvent);
}

void CMainWindow::MoveComputerBat()
{
	// Need to Move?
	if ((m_currGame.CompareNoCase(PELOTA_STR)==0)
		 && ((m_isFirstPlayerTurn)
		 	|| ((!m_isFirstPlayerTurn) && (m_ballXPos>m_firstBatXPosP2))))
	{		 	
		m_velP2 = 0;
		return;
	}
		
	// Avoid Auto-Goal
	if ((m_currGame.CompareNoCase(HOCKEY_STR)==0)
		&& (m_ballXPos>m_firstBatXPosP2+BAT_WIDTH))
	{
		if ((m_ballYPos>m_posP2)
			&& (m_ballYPos<m_posP2+m_batHeight))
		{
			// Away From Goal Area
			if (m_posP2<(m_winRect.Height()-m_batHeight)/2)
			{
				// Move Bat Up
				if (m_velP2>=0)
					m_velP2 = -BAT_INITIAL_SPEED;
			}
			else
			{
				// Move Bat Down
				if (m_velP2<=0)
					m_velP2 = BAT_INITIAL_SPEED;
			}
		}
		else
		{
			// Stop!
			m_velP2 = 0;
		}		
		return;
	}
		
	// Escape From Ball?
	if (((m_currGame.CompareNoCase(HOCKEY_STR)==0)
		&& (m_ballXPos<m_firstBatXPosP2)
		&& (m_ballXPos>m_secondBatXPosP2)
		&& (abs(m_ballXPos-m_secondBatXPosP2)
							<abs(m_ballXPos-m_firstBatXPosP2))))
	{
		// Escape From Ball
		if ((m_ballYVel<0)
			&& (m_ballYPos>m_posP2-m_ballWidth))
		{
			// Move Bat Down
			if (m_velP2<=0)
				m_velP2 = BAT_INITIAL_SPEED;
		}
		else
			if ((m_ballYVel>0)
				&& (m_ballYPos<m_posP2+m_batHeight))
			{
				// Move Bat Up
				if (m_velP2>=0)
					m_velP2 = -BAT_INITIAL_SPEED;
			}
			else
				m_velP2 = 0;
    }
    else
    {
		// Ball Tracking
		if (m_ballYPos<m_posP2)
		{
			// Move Bat Up
			if (m_velP2>=0)
				m_velP2 = -BAT_INITIAL_SPEED;
			}
			else
				if (m_ballYPos>(m_posP2+m_batHeight-m_ballWidth))
				{
					// Move Bat Down               
					if (m_velP2<=0)
						m_velP2 = BAT_INITIAL_SPEED;
				}
				else
				{
					// No Need to Move
					m_velP2 = 0;
				}
	}   	
}


/////////////////////////////////////////////////////////////////////////////
// Ball Functions

void CMainWindow::SetRandomBallPos()
{
	m_ballXPrevPos = 0;
	m_ballYPrevPos = 0;

	do {
	
		m_ballXPos = (int)(((double)rand())/RAND_MAX*(m_winRect.Width()));
		m_ballYPos = (int)(((double)rand())/RAND_MAX*(m_winRect.Height()));
	
	} while ((m_ballXPos<2*BAT_TO_MARGIN_SPACING)
			|| (m_ballXPos>(m_winRect.Width()-2*BAT_TO_MARGIN_SPACING-m_ballWidth))
			|| (m_ballYPos<MARGIN_WIDTH)
			|| (m_ballYPos>(m_winRect.Height()-MARGIN_WIDTH-m_ballWidth))
			
			// Out of Bat Y Position
			|| ((m_ballYPos>m_posP1-m_ballWidth)
				 && (m_ballYPos<m_posP1+m_batHeight))

			// Out Of Scores Area
			|| ((m_ballXPos>m_scoresRect.left-m_ballWidth)
				 && (m_ballXPos<m_scoresRect.right)
				 && (m_ballYPos>m_scoresRect.top-m_ballWidth)
				 && (m_ballYPos<m_scoresRect.bottom)));

	// Set Ball Speed
	int ballDir = (int)((double)rand()/RAND_MAX
						*(BALL_MAX_ANGLE-BALL_MIN_ANGLE)) + BALL_MIN_ANGLE;
	
	if (m_hiSpeed)
	{
		m_ballXVel = (int)(m_hiSpeedBallFactor*cos(ballDir*3.14/180));
		m_ballYVel = -(int)(m_hiSpeedBallFactor*sin(ballDir*3.14/180));
	}
	else
	{
		m_ballXVel = (int)(m_lowSpeedBallFactor*cos(ballDir*3.14/180));
		m_ballYVel = -(int)(m_lowSpeedBallFactor*sin(ballDir*3.14/180));
	}
	
	if (((double)rand())/RAND_MAX>0.5)
		m_ballYVel = -m_ballYVel;
		
	if (m_ballXPos>m_winRect.Width()/2)
		m_ballXVel = -m_ballXVel;
	
	if ((m_currGame.CompareNoCase(PELOTA_STR)==0)
		|| (m_currGame.CompareNoCase(TRAINING_STR)==0))
		m_ballXVel = - abs(m_ballXVel);
		
	m_ballEffect = 0;
	m_ballNextEffect = 0;
}	

void CMainWindow::CalculateBallPos()
{
	// Virtual Pos
	m_ballXPos += m_ballXVel;
	m_ballYPos += m_ballYVel;
			
	// Ball Out or Left Border Bounce?
	if (m_ballXPos<MARGIN_WIDTH)
	{
		if (m_currGame.CompareNoCase(PELOTA_STR)==0)
			LeftBounce(MARGIN_WIDTH);

		if (m_currGame.CompareNoCase(TENNIS_STR)==0)
			BallOut();

		if (m_currGame.CompareNoCase(HOCKEY_STR)==0)
		{
			if ((m_ballYPos>(m_winRect.Height()-GOAL_AREA_WIDTH)/2)
				&& (m_ballYPos<(m_winRect.Height()+GOAL_AREA_WIDTH)/2-m_ballWidth))
				BallOut();
			else
				LeftBounce(MARGIN_WIDTH);
		}

		if (m_currGame.CompareNoCase(TRAINING_STR)==0)
			LeftBounce(MARGIN_WIDTH);
	}

	// Ball Out or Right Border Bounce?
	if (m_ballXPos>m_winRect.Width()-MARGIN_WIDTH-m_ballWidth)
	{
		if (m_currGame.CompareNoCase(PELOTA_STR)==0)
			BallOut();

		if (m_currGame.CompareNoCase(TENNIS_STR)==0)
			BallOut();

		if (m_currGame.CompareNoCase(HOCKEY_STR)==0)
		{
			if ((m_ballYPos>(m_winRect.Height()-GOAL_AREA_WIDTH)/2)
				&& (m_ballYPos<(m_winRect.Height()+GOAL_AREA_WIDTH)/2-m_ballWidth))
				BallOut();
			else
				RightBounce(m_winRect.Width()-MARGIN_WIDTH);
		}

		if (m_currGame.CompareNoCase(TRAINING_STR)==0)
			BallOut();
	}

	// Horizontal Border Bounce?
    if (m_ballYPos<MARGIN_WIDTH)
    	TopBounce(MARGIN_WIDTH);
	
    if (m_ballYPos>m_winRect.Height()-MARGIN_WIDTH-m_ballWidth)
    	BottomBounce(m_winRect.Height()-MARGIN_WIDTH);
	
	// Player 1 Kick
	if ((m_firstBatXPosP1!=0)
		&& (m_ballXPos>m_firstBatXPosP1-m_ballWidth)
		&& (m_ballXPos<m_firstBatXPosP1+BAT_WIDTH)
		&& (m_ballYPos>m_posP1-m_ballWidth/2)
		&& (m_ballYPos<m_posP1+m_batHeight-m_ballWidth/2))
	{
		// When Playing Pelota Game, Player 1
		// Bat Works Only On Its Left Side...
		if ((m_currGame.CompareNoCase(PELOTA_STR)!=0)
		 	||	((m_currGame.CompareNoCase(PELOTA_STR)==0)
		 		 && (m_isFirstPlayerTurn) && m_ballXVel>0))
		 {
			if (m_effect)       
			{
				m_ballNextEffect = abs((int)(((double)m_velP1)/BAT_MAX_SPEED*100));
		    	
		    	// Calculate Rotation Direction
		    	if (((m_velP1>0) && (m_ballXVel>0))
		    		|| ((m_velP1<0) && (m_ballXVel<0)))

		    		m_ballNextEffect = -m_ballNextEffect;
		    }
				m_isFirstPlayerTurn = FALSE;

			if (m_ballXVel>0)
				RightBounce(m_firstBatXPosP1);
			else
				LeftBounce(m_firstBatXPosP1+BAT_WIDTH);
		}
	}
	
	if ((m_secondBatXPosP1!=0)
		&& (m_ballXPos>m_secondBatXPosP1-m_ballWidth)
		&& (m_ballXPos<m_secondBatXPosP1+BAT_WIDTH)
		&& (m_ballYPos>m_posP1-m_ballWidth/2)
		&& (m_ballYPos<m_posP1+m_batHeight-m_ballWidth/2))
	{
		if (m_effect)
		{
			m_ballNextEffect = abs((int)(((double)m_velP1)/BAT_MAX_SPEED*100));
		    	
	    	// Calculate Rotation Direction
	    	if (((m_velP1>0) && (m_ballXVel>0))
	    		|| ((m_velP1<0) && (m_ballXVel<0)))

	    		m_ballNextEffect = -m_ballNextEffect;
		}
				
		if (m_ballXVel>0)
			RightBounce(m_secondBatXPosP1);
		else
			LeftBounce(m_secondBatXPosP1+BAT_WIDTH);
	}
	
	// Player 2 Kick
	if ((m_firstBatXPosP2!=0)
		&& (m_ballXPos>m_firstBatXPosP2-m_ballWidth)
		&& (m_ballXPos<m_firstBatXPosP2+BAT_WIDTH)
		&& (m_ballYPos>m_posP2-m_ballWidth/2)
		&& (m_ballYPos<m_posP2+m_batHeight-m_ballWidth/2))
	{
		// When Playing Pelota Game, Player 2
		// Bat Works Only On Its Left Side...
		if ((m_currGame.CompareNoCase(PELOTA_STR)!=0)
		 	||	((m_currGame.CompareNoCase(PELOTA_STR)==0)
		 		 && (!m_isFirstPlayerTurn) && (m_ballXVel>0)))
		 {
			if (m_effect)
			{
				m_ballNextEffect = abs((int)(((double)m_velP2)/BAT_MAX_SPEED*100));
		    	
		    	// Calculate Rotation Direction
		    	if (((m_velP2>0) && (m_ballXVel>0))
		    		|| ((m_velP2<0) && (m_ballXVel<0)))

		    		m_ballNextEffect = -m_ballNextEffect;
		    }
		    
				m_isFirstPlayerTurn = TRUE;

			if (m_ballXVel>0)
				RightBounce(m_firstBatXPosP2);
			else
				LeftBounce(m_firstBatXPosP2+BAT_WIDTH);
		}
	}
	
	if ((m_secondBatXPosP2!=0)
		&& (m_ballXPos>m_secondBatXPosP2-m_ballWidth)
		&& (m_ballXPos<m_secondBatXPosP2+BAT_WIDTH)
		&& (m_ballYPos>m_posP2-m_ballWidth/2)
		&& (m_ballYPos<m_posP2+m_batHeight-m_ballWidth/2))
	{
		if (m_effect)
		{
			m_ballNextEffect = abs((int)(((double)m_velP2)/BAT_MAX_SPEED*100));
		    	
	    	// Calculate Rotation Direction
	    	if (((m_velP2>0) && (m_ballXVel>0))
	    		|| ((m_velP2<0) && (m_ballXVel<0)))

	    		m_ballNextEffect = -m_ballNextEffect;
	    }
		
		if (m_ballXVel>0)
			RightBounce(m_secondBatXPosP2);
		else
			LeftBounce(m_secondBatXPosP2+BAT_WIDTH);
	}
}
                                                          
void CMainWindow::BallOut()             
{
	// Stop Current Ball
	m_running = FALSE;
	KillTimer(TV_GAMES_TIMER_ID);
	
	// Hide Ball
	RedrawBall(TRUE);
	
	// Calculate Scores               
	if (m_currGame.CompareNoCase(PELOTA_STR)==0)
	{
		if (m_isFirstPlayerTurn)
			m_scoreP2++;
		else
			m_scoreP1++;
	}

	if (m_currGame.CompareNoCase(TENNIS_STR)==0)
	{
		if (m_ballXPrevPos<m_winRect.Width()/2)
			m_scoreP2++;
		else
			m_scoreP1++;
	}

	if (m_currGame.CompareNoCase(HOCKEY_STR)==0)
	{
		if (m_ballXPrevPos<m_winRect.Width()/2)
			m_scoreP2++;
		else
			m_scoreP1++;
	}

	if (m_currGame.CompareNoCase(TRAINING_STR)==0)
	{
		m_scoreP1++;
	}

	ShowScores();
	
	// End Game?
	if ((m_scoreP1==m_winnerScore)
		|| (m_scoreP2==m_winnerScore))
	{
		m_started = FALSE;
	}
	else
	{
		// Calculate Ball New Position
		SetRandomBallPos();					
	}
}

void CMainWindow::TopBounce(int nPos)
{
	if (m_sound)
		MessageBeep(-1);

	m_ballYPos += 2*(nPos - m_ballYPos);
	m_ballYVel = -m_ballYVel;
	
	if ((m_effect)
		&& (m_ballEffect!=0))
	{
		// Effect Feeling
		CalculateEffect();
    }

	// Update Effect Data
	m_ballEffect = m_ballNextEffect;
	m_ballNextEffect = 0;
}

void CMainWindow::LeftBounce(int nPos)
{
	if (m_sound)
		MessageBeep(-1);

	m_ballXPos += 2*(nPos - m_ballXPos);
	m_ballXVel = -m_ballXVel;
        
	if ((m_effect)
		&& (m_ballEffect!=0))
	{
		// Effect Feeling
		CalculateEffect();
	}

	// Update Effect Data
	m_ballEffect = m_ballNextEffect;
	m_ballNextEffect = 0;
}

void CMainWindow::RightBounce(int nPos)
{
	if (m_sound)
		MessageBeep(-1);

	m_ballXPos -= 2*(m_ballXPos + m_ballWidth - nPos);
	m_ballXVel = -m_ballXVel;

	if ((m_effect)
		&& (m_ballEffect!=0))
	{
		// Effect Feeling
		CalculateEffect();
	}

	// Update Effect Data
	m_ballEffect = m_ballNextEffect;
	m_ballNextEffect = 0;
}

void CMainWindow::BottomBounce(int nPos)
{
	if (m_sound)
		MessageBeep(-1);

	m_ballYPos -= 2*(m_ballYPos + m_ballWidth - nPos);
	m_ballYVel = -m_ballYVel;

	if ((m_effect)
		&& (m_ballEffect!=0))
	{
		// Effect Feeling
		CalculateEffect();
	}

	// Update Effect Data
	m_ballEffect = m_ballNextEffect;
	m_ballNextEffect = 0;
}
                                                          
void CMainWindow::CalculateEffect()
{
	if (m_ballEffect!=0)
	{
		double ballDir = atan(((double)-m_ballYVel)/m_ballXVel)*180/3.14;
		if (m_ballXVel<0)
			ballDir = ballDir + 180; 
			
		while (ballDir<0)
			ballDir += 360;
			
		ballDir += ((double)m_ballEffect)/100*m_effectAngle;
		
		// Ball Direction Control
		//    ballDir in [BALL_MIN_ANGLE, BALL_MAX_ANGLE]
		//    for any of the four arcs 0-90, 90-180, 180-270, 270-360
		int kPI = 0;
		while (kPI<360)
		{
			if ((ballDir>kPI)
				&& (ballDir<kPI + BALL_MIN_ANGLE))
			{
				ballDir = kPI + BALL_MIN_ANGLE;
				break;
			}
		
			if ((ballDir>kPI + BALL_MAX_ANGLE)
				&& (ballDir<kPI + 90))
			{
				ballDir = kPI + BALL_MAX_ANGLE;
				break;
			}
		
			kPI += 90;
		}
    	 
		if (m_hiSpeed)
		{
			m_ballXVel = (int)(m_hiSpeedBallFactor*cos(ballDir*3.14/180));
			m_ballYVel = -(int)(m_hiSpeedBallFactor*sin(ballDir*3.14/180));
		}
		else
		{
			m_ballXVel = (int)(m_lowSpeedBallFactor*cos(ballDir*3.14/180));
			m_ballYVel = -(int)(m_lowSpeedBallFactor*sin(ballDir*3.14/180));
		}
	}
}
 
 
/////////////////////////////////////////////////////////////////////////////
// Painting Management

void CMainWindow::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	
	CRect clientArea;
	GetClientRect(clientArea);

	// Select the Background Brush
	CBrush bkBrush;
	bkBrush.CreateSolidBrush(m_paper);
	CBrush* pOldBrush = dc.SelectObject(&bkBrush);
	
	// Paint the Background
	dc.Rectangle(clientArea);	
	
	dc.SelectObject(pOldBrush);
	bkBrush.DeleteObject();
	
	// Paint the Game Board
	CBrush inkBrush;
	inkBrush.CreateSolidBrush(m_ink);
		
	CRect border(clientArea);
	border.bottom = MARGIN_WIDTH;
		
	dc.FillRect(border,&inkBrush);
		
	border.bottom = clientArea.bottom;
	border.top = border.bottom - MARGIN_WIDTH;
		
	dc.FillRect(border,&inkBrush);
		
	if (m_currGame.CompareNoCase(PELOTA_STR)==0)
	{
		border = clientArea;
		border.right = MARGIN_WIDTH;			

		dc.FillRect(border,&inkBrush);
	}
		
	if (m_currGame.CompareNoCase(HOCKEY_STR)==0)
	{
		border = clientArea;
		border.right = MARGIN_WIDTH;
		border.bottom = (clientArea.Height() - GOAL_AREA_WIDTH)/2;			

		dc.FillRect(border,&inkBrush);

		border.right = clientArea.right;
		border.left = border.right - MARGIN_WIDTH;
			
		dc.FillRect(border,&inkBrush);
			
		border.top = border.bottom + GOAL_AREA_WIDTH;
		border.bottom = clientArea.bottom;			

		dc.FillRect(border,&inkBrush);

		border.left = 0;
		border.right = MARGIN_WIDTH;
	
		dc.FillRect(border,&inkBrush);
	}
		
	if (m_currGame.CompareNoCase(TRAINING_STR)==0)
	{
		border = clientArea;
		border.right = MARGIN_WIDTH;			

		dc.FillRect(border,&inkBrush);
	}
		
	// Write Scores
	if ((!m_running) || m_scoresAlwaysOnTop)
		ShowScores();
	
	// Free GDI Resources
	inkBrush.DeleteObject();
	
	// Redraw the Rackets
	m_prevPosP1 = 0;
	m_prevPosP2 = 0;
	RedrawBall();
	RedrawRackets(TRUE);
	ValidateRect(clientArea);	
}

void CMainWindow::RedrawRackets(BOOL bForceDrawing /*= FALSE*/)
{
	CRect clean;
	CRect batRect;
	CClientDC dc(this);
	
	CBrush inkBrush;
	inkBrush.CreateSolidBrush(m_ink);

	CBrush bkBrush;
	bkBrush.CreateSolidBrush(m_paper);
	
	// Draw Player 1 First Bat
	if ((m_firstBatXPosP1!=0)
		&& ((m_prevPosP1!=m_posP1) || (bForceDrawing)))
	{
		if (m_prevPosP1==0)
		{
			batRect.left = m_firstBatXPosP1;
			batRect.right = batRect.left + BAT_WIDTH;
			batRect.top = m_posP1;
			batRect.bottom = batRect.top + m_batHeight;

			dc.FillRect(batRect,&inkBrush);
			ValidateRect(batRect);
		}
		else
		{
			dc.BitBlt(m_firstBatXPosP1,m_posP1,BAT_WIDTH,m_batHeight,
							&dc,m_firstBatXPosP1,m_prevPosP1,SRCCOPY);
					  
			clean.left = m_firstBatXPosP1;
			clean.right = m_firstBatXPosP1 + BAT_WIDTH;
			clean.top = (m_prevPosP1<m_posP1) ? m_prevPosP1 : (m_posP1 + m_batHeight);
			clean.bottom = (m_prevPosP1<m_posP1) ? m_posP1 : (m_prevPosP1 + m_batHeight);
			
			dc.FillRect(clean,&bkBrush);    
			ValidateRect(clean);
		}
	}
	 	
	// Draw Player 1 Second Bat
	if ((m_secondBatXPosP1!=0)
		&& ((m_prevPosP1!=m_posP1) || (bForceDrawing)))
	{
		if (m_prevPosP1==0)
		{
			batRect.left = m_secondBatXPosP1;
			batRect.right = batRect.left + BAT_WIDTH;
			batRect.top = m_posP1;
			batRect.bottom = batRect.top + m_batHeight;
				
			dc.FillRect(batRect,&inkBrush);
			ValidateRect(batRect);
		}
		else
		{
			dc.BitBlt(m_secondBatXPosP1,m_posP1,BAT_WIDTH,m_batHeight,
							&dc,m_secondBatXPosP1,m_prevPosP1,SRCCOPY);
						  
			clean.left = m_secondBatXPosP1;
			clean.right = m_secondBatXPosP1 + BAT_WIDTH;
			clean.top = (m_prevPosP1<m_posP1) ? m_prevPosP1 : (m_posP1 + m_batHeight),
			clean.bottom = (m_prevPosP1<m_posP1) ? m_posP1 : (m_prevPosP1 + m_batHeight);

			dc.FillRect(clean,&bkBrush);    
			ValidateRect(clean);
		}
	}
			 	
	// Draw Player 2 First Bat
	if ((m_firstBatXPosP2!=0)
		&& ((m_prevPosP2!=m_posP2) || (bForceDrawing)))
    {
		if (m_prevPosP2==0)
		{
			batRect.left = m_firstBatXPosP2;
			batRect.right = batRect.left + BAT_WIDTH;
			batRect.top = m_posP2;
			batRect.bottom = batRect.top + m_batHeight;
			
			dc.FillRect(batRect,&inkBrush);
			ValidateRect(batRect);
		}       	
		else
		{
			dc.BitBlt(m_firstBatXPosP2,m_posP2,BAT_WIDTH,m_batHeight,
							&dc,m_firstBatXPosP2,m_prevPosP2,SRCCOPY);
					  
			clean.left =m_firstBatXPosP2;
			clean.top = (m_prevPosP2<m_posP2) ? m_prevPosP2 : (m_posP2 + m_batHeight);
			clean.right = m_firstBatXPosP2 + BAT_WIDTH;
			clean.bottom = (m_prevPosP2<m_posP2) ? m_posP2 : (m_prevPosP2 + m_batHeight);
		
			dc.FillRect(clean,&bkBrush);
			ValidateRect(clean);
		}
	}			
	// Draw Player 2 Second Bat
	if ((m_secondBatXPosP2!=0)
		&& ((m_prevPosP2!=m_posP2) || (bForceDrawing)))
	{
		if (m_prevPosP2==0)
		{
			batRect.left = m_secondBatXPosP2;
			batRect.right = batRect.left + BAT_WIDTH;
			batRect.top = m_posP2;
			batRect.bottom = batRect.top + m_batHeight;
			
			dc.FillRect(batRect,&inkBrush);
			ValidateRect(batRect);
		}
		else
		{
			dc.BitBlt(m_secondBatXPosP2,m_posP2,BAT_WIDTH,m_batHeight,
							&dc,m_secondBatXPosP2,m_prevPosP2,SRCCOPY);
					  
			CRect clean;
			clean.left = m_secondBatXPosP2;
			clean.right = m_secondBatXPosP2 + BAT_WIDTH;
			clean.top = (m_prevPosP2<m_posP2) ? m_prevPosP2 : (m_posP2 + m_batHeight);
			clean.bottom = (m_prevPosP2<m_posP2) ? m_posP2 : (m_prevPosP2 + m_batHeight);
			
			dc.FillRect(clean,&bkBrush);
			ValidateRect(clean);
		}
	}
	
	// Free GDI Resources
	bkBrush.DeleteObject();
	inkBrush.DeleteObject();
}                       

void CMainWindow::RedrawBall(BOOL bHideBall /*= FALSE*/)
{
	CRect ballRect;
	CClientDC dc(this);
	CBitmap* pOldBitmap = NULL;
	
	// Create an Auxiliary DC
	CDC memDC;
	memDC.CreateCompatibleDC(&dc);
	
	CBrush inkBrush;
	inkBrush.CreateSolidBrush(m_ink);

	CBrush bkBrush;
	bkBrush.CreateSolidBrush(m_paper);
		
	// Hide Ball
	if ((m_ballBmp.m_hObject!=NULL)
		&& (m_ballXPrevPos!=0)
		&& (m_ballYPrevPos!=0) && (bHideBall))
	{                             
		// Restore Ball Background
		pOldBitmap = memDC.SelectObject(&m_ballBmp);

		dc.BitBlt(m_ballXPrevPos,m_ballYPrevPos,
					m_ballWidth,m_ballWidth,&memDC,0,0,SRCCOPY);
	}
        
	// Draw Ball
	if ((m_ballXPos!=0) && (m_ballYPos!=0) && (!bHideBall))
	{
		// Save Ball Background
    	if (m_ballBmp.m_hObject!=NULL)
    		m_ballBmp.DeleteObject();
    		
		m_ballBmp.CreateCompatibleBitmap(&dc,MAX_BALL_WIDTH,MAX_BALL_WIDTH);
		pOldBitmap = memDC.SelectObject(&m_ballBmp);
		
		// Copy to m_ballBmp
		memDC.BitBlt(0,0,m_ballWidth,m_ballWidth,
						&dc,m_ballXPos,m_ballYPos,SRCCOPY);
            
		ballRect.left = m_ballXPos;
		ballRect.right = ballRect.left + m_ballWidth;
		ballRect.top = m_ballYPos;
		ballRect.bottom = ballRect.top + m_ballWidth;
			
		dc.FillRect(ballRect,&inkBrush);
		ValidateRect(ballRect);
	}

	// Free GDI Resources
	inkBrush.DeleteObject();
	bkBrush.DeleteObject();

	if (pOldBitmap!=NULL)
		memDC.SelectObject(pOldBitmap);
	memDC.DeleteDC();
}

void CMainWindow::ShowScores()
{
	// Delete Previous Scores
	HideScores();
		
	// Write Player 1 Score
	if (m_scoreP1>9)
		DrawScoreDigit(1,m_scoresRect.left-2*SCORES_CHAR_WIDTH,m_scoresRect.top);

	DrawScoreDigit(m_scoreP1%10,
					m_scoresRect.left+2*SCORES_CHAR_WIDTH,m_scoresRect.top);
	
	if (m_currGame.CompareNoCase(TRAINING_STR)!=0)
	{
		// Write Player 2 Score
		if (m_scoreP2>9)
			DrawScoreDigit(1,
						m_scoresRect.right-7*SCORES_CHAR_WIDTH,m_scoresRect.top);
	
		DrawScoreDigit(m_scoreP2%10,
						m_scoresRect.right-3*SCORES_CHAR_WIDTH,m_scoresRect.top);
	}
	
    m_areScoresShown = TRUE;
	
	ValidateRect(m_scoresRect);
}

void CMainWindow::DrawScoreDigit(int digit, int xCoor, int yCoor)
{
	CClientDC dc(this);
	
	CRect rect;
	CBrush inkBrush;
	inkBrush.CreateSolidBrush(m_ink);
	
	// "a" Segment
	if ((digit!=1) && (digit!=4))
	{
		rect.left = xCoor;
		rect.right = rect.left + 3*SCORES_CHAR_WIDTH;
		rect.top = yCoor;
		rect.bottom = rect.top + SCORES_CHAR_WIDTH;
		
		dc.FillRect(rect,&inkBrush);
	}
	
	// "b" Segment
	if ((digit!=5) && (digit!=6))
	{
		rect.left = xCoor + 2*SCORES_CHAR_WIDTH;
		rect.right = rect.left + SCORES_CHAR_WIDTH;
		rect.top = yCoor;
		rect.bottom = rect.top + 3*SCORES_CHAR_WIDTH;
		
		dc.FillRect(rect,&inkBrush);
	}
	
	// "c" Segment
	if (digit!=2)
	{
		rect.left = xCoor + 2*SCORES_CHAR_WIDTH;
		rect.right = rect.left + SCORES_CHAR_WIDTH;
		rect.top = yCoor + 2*SCORES_CHAR_WIDTH;
		rect.bottom = rect.top + 3*SCORES_CHAR_WIDTH;
		
		dc.FillRect(rect,&inkBrush);
	}
	
	// "d" Segment
	if ((digit!=1) && (digit!=4) && (digit!=7))
	{
		rect.left = xCoor;
		rect.right = rect.left + 3*SCORES_CHAR_WIDTH;
		rect.top = yCoor + 4*SCORES_CHAR_WIDTH;
		rect.bottom = rect.top + SCORES_CHAR_WIDTH;
		
		dc.FillRect(rect,&inkBrush);
	}
	
	// "e" Segment
	if ((digit==0) || (digit==2) || (digit==6) || (digit==8))
	{
		rect.left = xCoor;
		rect.right = rect.left + SCORES_CHAR_WIDTH;
		rect.top = yCoor + 2*SCORES_CHAR_WIDTH;
		rect.bottom = rect.top + 3*SCORES_CHAR_WIDTH;
		
		dc.FillRect(rect,&inkBrush);
	}
	
	// "f" Segment
	if ((digit!=1) && (digit!=2) && (digit!=3) && (digit!=7))
	{
		rect.left = xCoor;
		rect.right = rect.left + SCORES_CHAR_WIDTH;
		rect.top = yCoor;
		rect.bottom = rect.top + 3*SCORES_CHAR_WIDTH;
		
		dc.FillRect(rect,&inkBrush);
	}
	
	// "g" Segment
	if ((digit!=0) && (digit!=1) && (digit!=7))
	{
		rect.left = xCoor;
		rect.right = rect.left + 3*SCORES_CHAR_WIDTH;
		rect.top = yCoor + 2*SCORES_CHAR_WIDTH;
		rect.bottom = rect.top + SCORES_CHAR_WIDTH;
		
		dc.FillRect(rect,&inkBrush);
	}
	
	// Free GDI Resources
	inkBrush.DeleteObject();
}
 
void CMainWindow::HideScores()
{
	CClientDC dc(this);
	
	// Delete Scores
	CBrush bkBrush;
	bkBrush.CreateSolidBrush(m_paper);
	
	dc.FillRect(m_scoresRect,&bkBrush);
	bkBrush.DeleteObject();
	
    m_areScoresShown = FALSE;

	ValidateRect(m_scoresRect);
}
	
/////////////////////////////////////////////////////////////////////////////
// CMainWindow message map:
BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
	//{{AFX_MSG_MAP( CMainWindow )
	ON_COMMAND(ID_GAME_NEW, OnGameNew)
	ON_COMMAND(ID_GAME_PAUSE, OnGamePause)
	ON_COMMAND(ID_GAME_PELOTA, OnGamePelota)
	ON_COMMAND(ID_GAME_TENNIS, OnGameTennis)
	ON_COMMAND(ID_GAME_HOCKEY, OnGameHockey)
	ON_COMMAND(ID_GAME_TRAINING, OnGameTraining)
	ON_COMMAND(ID_GAME_COLORS, OnGameColors)
	ON_COMMAND(ID_GAME_PLAYERS_HUMAN, OnGamePlayersHuman)
	ON_COMMAND(ID_GAME_PLAYERS_COMPUTER, OnGamePlayersComputer)
	ON_COMMAND(ID_GAME_KEYBOARD, OnGameKeyboard)
	ON_COMMAND(ID_GAME_SCORESALWAYSONTOP, OnGameScoresalwaysontop)
	ON_COMMAND(ID_GAME_SOUND, OnGameSound)
	ON_COMMAND(ID_GAME_HIGHSPEED, OnGameHighspeed)
	ON_COMMAND(ID_GAME_BALLEFFECT, OnGameBalleffect)
	ON_COMMAND(ID_GAME_ACCELERATION, OnGameAcceleration)
	ON_COMMAND(ID_GAME_LARGERACKETS, OnGameLargerackets)
	ON_COMMAND(ID_GAME_ADVANCEDOPTIONS, OnGameAdvancedoptions)
	ON_UPDATE_COMMAND_UI(ID_GAME_PAUSE, OnUpdateGamePause)
	ON_UPDATE_COMMAND_UI(ID_GAME_PLAYERS_HUMAN, OnUpdateGamePlayersHuman)
	ON_UPDATE_COMMAND_UI(ID_GAME_PLAYERS_COMPUTER, OnUpdateGamePlayersComputer)
	ON_UPDATE_COMMAND_UI(ID_GAME_SCORESALWAYSONTOP, OnUpdateGameScoresalwaysontop)
	ON_UPDATE_COMMAND_UI(ID_GAME_SOUND, OnUpdateGameSound)
	ON_UPDATE_COMMAND_UI(ID_GAME_HIGHSPEED, OnUpdateGameHighspeed)
	ON_UPDATE_COMMAND_UI(ID_GAME_BALLEFFECT, OnUpdateGameBalleffect)
	ON_UPDATE_COMMAND_UI(ID_GAME_ACCELERATION, OnUpdateGameAcceleration)
	ON_UPDATE_COMMAND_UI(ID_GAME_LARGERACKETS, OnUpdateGameLargerackets)
	ON_UPDATE_COMMAND_UI(ID_GAME_PELOTA, OnUpdateGamePelota)
	ON_UPDATE_COMMAND_UI(ID_GAME_TENNIS, OnUpdateGameTennis)
	ON_UPDATE_COMMAND_UI(ID_GAME_HOCKEY, OnUpdateGameHockey)
	ON_UPDATE_COMMAND_UI(ID_GAME_TRAINING, OnUpdateGameTraining)
	ON_COMMAND(ID_HELP_OPTIONS, OnHelpOptions)
	ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
	ON_WM_PAINT()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_TIMER()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_SIZE()
	ON_WM_MENUSELECT()
	ON_WM_MOVE()
	ON_WM_WINDOWPOSCHANGING()
	ON_WM_NCLBUTTONDBLCLK()
	ON_WM_NCLBUTTONDOWN()
	ON_WM_SYSCOMMAND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Game Menu Management

void CMainWindow::OnGameNew()
{
	m_paused = FALSE;
	m_started = TRUE;
	m_running = FALSE;       
	KillTimer(TV_GAMES_TIMER_ID);
	
	// Set Bats Position
	m_posP1 = (m_winRect.Height() - m_batHeight)/2;
	m_prevPosP1 = 0;
	m_velP1 = 0;

	m_posP2 = m_posP1;
	m_prevPosP2 = 0;
	m_velP2 = 0;

	// Set Ball Position
	SetRandomBallPos();

	// Reset Scores
	m_scoreP1 = 0;
	m_scoreP2 = 0;
	
	// Redraw Window
	Invalidate(FALSE);
	UpdateWindow();
	
	// Write "0 - 0" Score
	ShowScores();
}

void CMainWindow::OnGamePause()
{
	if (m_paused)
	{
		m_paused = FALSE;                    
		m_running = TRUE;
		SetTimer(TV_GAMES_TIMER_ID,TV_GAMES_TIMER_DELAY,NULL);
		SetWindowText(m_appName);
	}                                              
	else
	{
		m_paused = TRUE;
		m_running = FALSE;
		KillTimer(TV_GAMES_TIMER_ID);
		
		CString caption = m_appName;
		caption += PAUSED_GAME_STR;
		SetWindowText(caption);
	}
}

void CMainWindow::OnUpdateGamePause(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_paused);
}

void CMainWindow::OnGamePelota()
{
	m_currGame = PELOTA_STR;
	m_isFirstPlayerTurn = TRUE;
	                   
	m_firstBatXPosP2 = m_winRect.Width() - BAT_TO_MARGIN_SPACING - BAT_WIDTH;
	m_secondBatXPosP2 = 0;
	m_firstBatXPosP1 = m_firstBatXPosP2 - BAT_TO_BAT_MIN_SPACING - BAT_WIDTH;
	m_secondBatXPosP1 = 0;
	
	OnGameNew();
}

void CMainWindow::OnUpdateGamePelota(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_currGame.CompareNoCase(PELOTA_STR)==0);
}

void CMainWindow::OnGameTennis()
{
	m_currGame = TENNIS_STR;

	m_firstBatXPosP1 = BAT_TO_MARGIN_SPACING;
	m_secondBatXPosP1 = 0;
	m_firstBatXPosP2 = m_winRect.right - BAT_TO_MARGIN_SPACING - BAT_WIDTH;
	m_secondBatXPosP2 = 0;
	
	OnGameNew();
}

void CMainWindow::OnUpdateGameTennis(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_currGame.CompareNoCase(TENNIS_STR)==0);
}

void CMainWindow::OnGameHockey()
{
	m_currGame = HOCKEY_STR;

	m_firstBatXPosP1 = BAT_TO_MARGIN_SPACING;
	m_firstBatXPosP2 = m_winRect.right - BAT_TO_MARGIN_SPACING - BAT_WIDTH;
	m_secondBatXPosP1 = m_firstBatXPosP2 - BAT_TO_BAT_MAX_SPACING;
	m_secondBatXPosP2 = m_firstBatXPosP1 + BAT_TO_BAT_MAX_SPACING;
	
	OnGameNew();
}

void CMainWindow::OnUpdateGameHockey(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_currGame.CompareNoCase(HOCKEY_STR)==0);
}

void CMainWindow::OnGameTraining()
{
	m_currGame = TRAINING_STR;

	m_firstBatXPosP1 = m_winRect.Width() - BAT_TO_MARGIN_SPACING - BAT_WIDTH;
	m_secondBatXPosP1 = 0;
	m_firstBatXPosP2 = 0;
	m_secondBatXPosP2 = 0;

	OnGameNew();
}

void CMainWindow::OnUpdateGameTraining(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_currGame.CompareNoCase(TRAINING_STR)==0);
}

void CMainWindow::OnGameColors()
{
	CColorsDlg dlg(this);
	
	dlg.m_selInk = GetColorName(m_ink);
	dlg.m_selPaper = GetColorName(m_paper);
	
	if (dlg.DoModal()==IDOK)
	{
		m_ink = GetColorValue(dlg.m_selInk);
		m_paper = GetColorValue(dlg.m_selPaper);

		// Repaint
		Invalidate(NULL);
		UpdateWindow();
	}
}

void CMainWindow::OnGamePlayersComputer()
{
	CComputerSkillDlg dlg(this);
	dlg.m_computerSkill = m_computerSpeed;

	dlg.DoModal();
	
	m_computerSpeed = dlg.m_computerSkill;
}

void CMainWindow::OnUpdateGamePlayersComputer(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_computerSpeed!=COMPUTER_NOT_PLAYING);
}

void CMainWindow::OnGamePlayersHuman()
{
	// Disable Computer 2nd Player Control
	m_computerSpeed = COMPUTER_NOT_PLAYING;
}

void CMainWindow::OnUpdateGamePlayersHuman(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_computerSpeed==COMPUTER_NOT_PLAYING);
}

void CMainWindow::OnGameKeyboard()
{
	CKeyboardDlg dlg;
	dlg.m_p1Up = m_upKeyP1;
	dlg.m_p1Down = m_downKeyP1;
	dlg.m_p2Up = m_upKeyP2;
	dlg.m_p2Down = m_downKeyP2;
	
	if (dlg.DoModal()==IDOK)
	{
		m_upKeyP1 = dlg.m_p1Up;
		m_downKeyP1 = dlg.m_p1Down;
		m_upKeyP2 = dlg.m_p2Up;
		m_downKeyP2 = dlg.m_p2Down;
	}
}

void CMainWindow::OnGameScoresalwaysontop()
{
	m_scoresAlwaysOnTop = !m_scoresAlwaysOnTop;

	if (m_scoresAlwaysOnTop)
		ShowScores();
	else
		if (m_running)
			HideScores();
}

void CMainWindow::OnUpdateGameScoresalwaysontop(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_scoresAlwaysOnTop);
}

void CMainWindow::OnGameSound()
{
	m_sound = !m_sound;
}

void CMainWindow::OnUpdateGameSound(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_sound);
}

void CMainWindow::OnGameHighspeed()
{
	m_hiSpeed = !m_hiSpeed;

	// Set Ball Speed
	int ballDir = (int)((double)rand()/RAND_MAX*30) + 30;	// [30..60]
	
	if (m_hiSpeed)
	{
		m_ballXVel = (int)(m_hiSpeedBallFactor*cos(ballDir*3.14/180));
		m_ballYVel = (int)(m_hiSpeedBallFactor*sin(ballDir*3.14/180));
	}
	else
	{
		m_ballXVel = (int)(m_lowSpeedBallFactor*cos(ballDir*3.14/180));
		m_ballYVel = (int)(m_lowSpeedBallFactor*sin(ballDir*3.14/180));
	}
	
	if ((double)rand()/RAND_MAX>0.5)
		m_ballYVel = -m_ballYVel;
		
	if (m_ballXPos>m_winRect.Width()/2)
		m_ballXVel = -m_ballXVel;
}

void CMainWindow::OnUpdateGameHighspeed(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_hiSpeed);
}

void CMainWindow::OnGameBalleffect()
{
	m_effect = !m_effect;
}

void CMainWindow::OnUpdateGameBalleffect(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_effect);
}

void CMainWindow::OnGameAcceleration()
{
	m_accelerated = !m_accelerated;
}

void CMainWindow::OnUpdateGameAcceleration(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_accelerated);
}          

void CMainWindow::OnGameLargerackets()
{
	m_largeBats = !m_largeBats;
	m_batHeight = m_largeBats ? m_bigBatHeight : m_smallBatHeight;

	// Redraw Game Board
	Invalidate(FALSE);
	UpdateWindow();
}

void CMainWindow::OnUpdateGameLargerackets(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_largeBats);
}

void CMainWindow::OnGameAdvancedoptions()
{
	COptionsDlg dlg(this);
	
	dlg.m_currGame = m_currGame;
	
	dlg.m_sound = m_sound;
	dlg.m_effect = m_effect;
	dlg.m_hiSpeed = m_hiSpeed;
	dlg.m_largeBats = m_largeBats;
	dlg.m_accelerated = m_accelerated;
	dlg.m_scoresAlwaysOnTop = m_scoresAlwaysOnTop;

	dlg.m_winnerScore = m_winnerScore;
	dlg.m_loSpeedValue = m_lowSpeedBallFactor;
	dlg.m_hiSpeedValue = m_hiSpeedBallFactor;
	dlg.m_bigBatHeight = m_bigBatHeight;
	dlg.m_smallBatHeight = m_smallBatHeight;
	dlg.m_ballWidth = m_ballWidth;
	dlg.m_maxEffect = m_effectAngle;
	
	dlg.m_ink = m_ink;
	dlg.m_paper = m_paper;
	
	dlg.m_upKeyP1 = m_upKeyP1;
	dlg.m_downKeyP1 = m_downKeyP1;
	dlg.m_upKeyP2 = m_upKeyP2;
	dlg.m_downKeyP2 = m_downKeyP2;
	
	dlg.m_computerSpeed = m_computerSpeed;
	
	if (dlg.DoModal()==IDOK)
	{
		// Restore Parameters
		m_sound = dlg.m_sound;
		m_effect = dlg.m_effect;
		m_hiSpeed = dlg.m_hiSpeed;
		m_largeBats = dlg.m_largeBats;
		m_accelerated = dlg.m_accelerated;
		m_scoresAlwaysOnTop = dlg.m_scoresAlwaysOnTop;

		m_winnerScore = dlg.m_winnerScore;
		m_lowSpeedBallFactor = dlg.m_loSpeedValue;
		m_hiSpeedBallFactor = dlg.m_hiSpeedValue;
		m_bigBatHeight = dlg.m_bigBatHeight;
		m_smallBatHeight = dlg.m_smallBatHeight;
		m_ballWidth = dlg.m_ballWidth;
		m_effectAngle = dlg.m_maxEffect;
	
		m_ink = dlg.m_ink;
		m_paper = dlg.m_paper;
	
		m_upKeyP1 = dlg.m_upKeyP1;
		m_downKeyP1 = dlg.m_downKeyP1;
		m_upKeyP2 = dlg.m_upKeyP2;
		m_downKeyP2 = dlg.m_downKeyP2;
	
		m_computerSpeed = dlg.m_computerSpeed;

		// Set Bats Height
		m_batHeight = m_largeBats ? m_bigBatHeight : m_smallBatHeight;

		// Set Current Game
		if (m_currGame!=dlg.m_currGame)
		{
			m_currGame = dlg.m_currGame;

			if (m_currGame.CompareNoCase(PELOTA_STR)==0)
				OnGamePelota();

			if (m_currGame.CompareNoCase(TENNIS_STR)==0)
				OnGameTennis();

			if (m_currGame.CompareNoCase(HOCKEY_STR)==0)
				OnGameHockey();

			if (m_currGame.CompareNoCase(TRAINING_STR)==0)
				OnGameTraining();
		}

		// Window Repainting
		Invalidate(FALSE);
		UpdateWindow();
	}
}


/////////////////////////////////////////////////////////////////////////////
// Help Menu Management

void CMainWindow::OnHelpOptions()
{
	CDialog dlg(IDD_OPTIONSHELP,this);
	
	dlg.DoModal();
}

void CMainWindow::OnHelpAbout()
{
	CDialog dlg("ABOUTBOX",this);
	
	dlg.DoModal();
}


/////////////////////////////////////////////////////////////////////////////
// Mouse Management

void CMainWindow::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	// Do Nothing During the Game...
	if (!m_running)
		OnGameAdvancedoptions();
}

void CMainWindow::OnNcLButtonDblClk(UINT nHitTest, CPoint point)
{
	// Suspend Game
	if (m_running)
		OnGamePause();
		
	CFrameWnd::OnNcLButtonDblClk(nHitTest, point);
}

void CMainWindow::OnNcLButtonDown(UINT nHitTest, CPoint point)
{
	// Suspend Game
	if (m_running)
		OnGamePause();
		
	CFrameWnd::OnNcLButtonDown(nHitTest, point);
}



/////////////////////////////////////////////////////////////////////////////
// Miscellaneous

void CMainWindow::OnMenuSelect(UINT nItemID, UINT nFlags, HMENU hSysMenu)
{
	// Suspend Game
	if (m_running)
		OnGamePause();
		
	CFrameWnd::OnMenuSelect(nItemID, nFlags, hSysMenu);
}

void CMainWindow::OnSize(UINT nType, int cx, int cy)
{
	if ((nType==SIZE_MINIMIZED)
		&& (m_running))
		OnGamePause();
		
	if (nType!=SIZE_RESTORED)
		CFrameWnd::OnSize(nType, cx, cy);
}

void CMainWindow::OnMove(int x, int y)
{
	// No Move
//	CFrameWnd::OnMove(x, y);
}
 
void CMainWindow::OnWindowPosChanging(WINDOWPOS FAR* lpwndpos)
{
static BOOL m_wndCreation = TRUE;

	// No Resizing neither Moving   
	if (m_wndCreation)
		m_wndCreation = FALSE;
	else               
	{       
		CClientDC dc(this);
		int vertRes = dc.GetDeviceCaps(VERTRES);
		int horzRes = dc.GetDeviceCaps(HORZRES);
		
		if ((IsZoomed())
			&& ((lpwndpos->x!=-1)
				|| (lpwndpos->y!=-1)
				|| (lpwndpos->cx<horzRes)
				|| (lpwndpos->cy<vertRes)))
			lpwndpos->flags |= (SWP_NOMOVE|SWP_NOSIZE);
    }
    
	CFrameWnd::OnWindowPosChanging(lpwndpos);
}


void CMainWindow::OnSysCommand(UINT nID, LPARAM lParam)
{
	if (nID==SC_TASKLIST) 
	{
		// Suspend Game
		if (m_running)
			OnGamePause();
			
		AfxMessageBox("Task List Manager not available.",MB_ICONEXCLAMATION);
		return;
	}
	
	CFrameWnd ::OnSysCommand(nID, lParam);
}





/////////////////////////////////////////////////////////////////////////////
// CTheApp

// InitInstance:
// When any CTheApp object is created, this member function is automatically
// called.  Any data may be set up at this point.
//
// Also, the main window of the application should be created and shown here.
// Return TRUE if the initialization is successful.
//
BOOL CTheApp::InitInstance()
{
	SetDialogBkColor();     // hook gray dialogs (was default in MFC V1)

	m_pMainWnd = new CMainWindow();

	CMainWindow* pWnd = (CMainWindow*)m_pMainWnd;

	// Load INI Data
	pWnd->m_currGame = GetProfileString(CONFIG_SECTION,GAME_TYPE,DEF_GAME_TYPE);
	pWnd->m_sound = GetProfileInt(CONFIG_SECTION,SOUND,DEF_SOUND);
	pWnd->m_hiSpeed = GetProfileInt(CONFIG_SECTION,HIGH_SPEED,DEF_HIGH_SPEED);
	pWnd->m_largeBats = GetProfileInt(CONFIG_SECTION,LARGE_BATS,DEF_LARGE_BATS);
	pWnd->m_effect = GetProfileInt(CONFIG_SECTION,BALL_EFFECT,DEF_BALL_EFFECT);
	pWnd->m_accelerated = GetProfileInt(CONFIG_SECTION,ACCELERATED,DEF_ACCELERATED);
	pWnd->m_ink = GetColorValue(GetProfileString(CONFIG_SECTION,INK_COLOR,DEF_INK_COLOR));
	pWnd->m_paper = GetColorValue(GetProfileString(CONFIG_SECTION,PAPER_COLOR,DEF_PAPER_COLOR));
	pWnd->m_scoresAlwaysOnTop = GetProfileInt(CONFIG_SECTION,SCORES_ON_TOP,DEF_SCORES_ON_TOP);

	pWnd->m_winnerScore = GetProfileInt(PARAM_SECTION,WINNER_SCORE,DEF_WINNER_SCORE);
	pWnd->m_lowSpeedBallFactor = GetProfileInt(PARAM_SECTION,LOW_SPEED_VALUE,DEF_LOW_SPEED_VALUE);
	pWnd->m_hiSpeedBallFactor = GetProfileInt(PARAM_SECTION,HIGH_SPEED_VALUE,DEF_HIGH_SPEED_VALUE);
	pWnd->m_bigBatHeight = GetProfileInt(PARAM_SECTION,BIG_BAT_HEIGHT,DEF_BIG_BAT_HEIGHT);
	pWnd->m_smallBatHeight = GetProfileInt(PARAM_SECTION,SMALL_BAT_HEIGHT,DEF_SMALL_BAT_HEIGHT);
	pWnd->m_ballWidth = GetProfileInt(PARAM_SECTION,BALL_WIDTH,DEF_BALL_WIDTH);
	pWnd->m_computerSpeed = GetProfileInt(PARAM_SECTION,COMPUTER_SPEED,DEF_COMPUTER_SPEED);
	pWnd->m_upKeyP1 = (UINT)GetProfileInt(PARAM_SECTION,PLAYER1_UP_KEY,DEF_PLAYER1_UP_KEY);
	pWnd->m_downKeyP1 = (UINT)GetProfileInt(PARAM_SECTION,PLAYER1_DOWN_KEY,DEF_PLAYER1_DOWN_KEY);
	pWnd->m_upKeyP2 = (UINT)GetProfileInt(PARAM_SECTION,PLAYER2_UP_KEY,DEF_PLAYER2_UP_KEY);
	pWnd->m_downKeyP2 = (UINT)GetProfileInt(PARAM_SECTION,PLAYER2_DOWN_KEY,DEF_PLAYER2_DOWN_KEY);
	pWnd->m_effectAngle = (UINT)GetProfileInt(PARAM_SECTION,EFFECT_ANGOLATION,DEF_EFFECT_ANGOLATION);
	
	m_pMainWnd->ShowWindow(SW_SHOW);
	m_pMainWnd->UpdateWindow();

	pWnd->InitMainWindow();
	m_pszAppName = pWnd->m_appName;	

	return TRUE;
}

 
 

 
/////////////////////////////////////////////////////////////////////////////
// theApp:
// Just creating this application object runs the whole application.

CTheApp NEAR theApp;





