/////////////////////////////////////////////////////////////////////////////
// tvgames.cpp: application declaration file

#ifndef __TVGAMES_H__
#define __TVGAMES_H__
                                    
                                    
// Main Window Macros
#define	TV_GAMES_TIMER_ID		0x01            
#define	TV_GAMES_TIMER_DELAY	80

// Kinetic Macros
#define BAT_INITIAL_SPEED		20
#define BAT_SPEED_RATIO			10
#define BAT_MAX_SPEED			50
#define	COMPUTER_NOT_PLAYING	0
#define	COMPUTER_HI_SPEED		1
#define	COMPUER_MED_SPEED		2
#define COMPUTER_LO_SPEED		4
#define BALL_MAX_ANGLE	    	60
#define BALL_MIN_ANGLE			10
    

// Game Macros
#define BAT_WIDTH				30
#define MARGIN_WIDTH			15
#define GOAL_AREA_WIDTH			160
#define BAT_TO_MARGIN_SPACING	60
#define BAT_TO_BAT_MIN_SPACING	60	
#define BAT_TO_BAT_MAX_SPACING	120	
#define SCORES_CHAR_WIDTH		15
#define SCORES_VERT_SPACING		15
#define SCORES_HORZ_SPACING		45


// Color Macros
#define BLACK			RGB(0x00,0x00,0x00)
#define WHITE			RGB(0xff,0xff,0xff)
#define DARKGREY		RGB(0x80,0x80,0x80)
#define GREY			RGB(0xc0,0xc0,0xc0)
#define BROWN			RGB(0x80,0x00,0x00)
#define RED				RGB(0xff,0x00,0x00)
#define YELLOW			RGB(0x80,0x80,0x00)
#define LIGHTYELLOW		RGB(0xff,0xff,0x00)
#define GREEN			RGB(0x00,0x80,0x00)
#define LIGHTGREEN		RGB(0x00,0xff,0x00)
#define CYAN			RGB(0x00,0x80,0x80)
#define LIGHTCYAN		RGB(0x00,0xff,0xff)
#define BLUE			RGB(0x00,0x00,0x80)
#define LIGHTBLUE		RGB(0x00,0x00,0xff)
#define MAGENTA			RGB(0x80,0x00,0x80)
#define FUCSIA			RGB(0xff,0x00,0xff)


// Game Idenitifers
#define	PELOTA_STR		"Pelota"
#define	TENNIS_STR		"Tennis"
#define	HOCKEY_STR		"Hockey"
#define	TRAINING_STR	"Training"          

#define	PAUSED_GAME_STR	" - [Paused]"


// .INI Sections & Entries
#define CONFIG_SECTION		"Configuration"
#define	GAME_TYPE			"Game"
#define	SOUND				"Sound"
#define	HIGH_SPEED			"High Speed"
#define	LARGE_BATS			"Large Bats"
#define	BALL_EFFECT			"Ball Effect"
#define	ACCELERATED			"Acceleration"
#define	INK_COLOR			"Ink Color"
#define	PAPER_COLOR			"Paper Color"
#define	SCORES_ON_TOP		"Scores Always On Top"        

#define PARAM_SECTION		"Parameters"
#define WINNER_SCORE		"Winner Score"
#define	LOW_SPEED_VALUE		"Lo-Speed Ball Factor"
#define	HIGH_SPEED_VALUE	"Hi-Speed Ball Factor"
#define BIG_BAT_HEIGHT		"Big Bat"
#define	SMALL_BAT_HEIGHT	"Small Bat"
#define BALL_WIDTH			"Ball Width"
#define COMPUTER_SPEED		"Computer Speed"
#define PLAYER1_UP_KEY		"Player1 Up Key"
#define PLAYER1_DOWN_KEY	"Player1 Down Key"
#define PLAYER2_UP_KEY		"Player2 Up Key"
#define PLAYER2_DOWN_KEY	"Player2 Down Key"
#define	EFFECT_ANGOLATION	"Max Effect Angolation"


// Default Values
#define	DEF_GAME_TYPE			TENNIS_STR
#define	DEF_SOUND				1
#define	DEF_HIGH_SPEED			0
#define	DEF_LARGE_BATS			1
#define	DEF_BALL_EFFECT			0
#define	DEF_ACCELERATED			0
#define	DEF_INK_COLOR			"Grey"
#define	DEF_PAPER_COLOR			"DarkGrey"
#define	DEF_SCORES_ON_TOP		0    
    
#define	DEF_WINNER_SCORE		15
#define	DEF_LOW_SPEED_VALUE		25
#define	DEF_HIGH_SPEED_VALUE	40
#define DEF_BALL_WIDTH			15
#define DEF_BIG_BAT_HEIGHT		100
#define	DEF_SMALL_BAT_HEIGHT	60
#define DEF_COMPUTER_SPEED		COMPUER_MED_SPEED
#define DEF_PLAYER1_UP_KEY		0x41
#define DEF_PLAYER1_DOWN_KEY	0x5a	
#define DEF_PLAYER2_UP_KEY		0x26		
#define DEF_PLAYER2_DOWN_KEY	0x28	
#define	DEF_EFFECT_ANGOLATION	30

        
// Parameter Limits
#define MIN_SPEED_VALUE			1
#define MAX_SPEED_VALUE			50
#define MIN_BAT_HEIGHT			10
#define MAX_BAT_HEIGHT			100
#define MIN_BALL_WIDTH			10
#define MAX_BALL_WIDTH			30
#define MAX_BALL_EFFECT			30


/////////////////////////////////////////////////////////////////////////////
// Global Functions Declaration
CString GetColorName(COLORREF color);
COLORREF GetColorValue(const char* colorName);

CString GetVirtualKeyName(UINT nVKCode);


/////////////////////////////////////////////////////////////////////////////

// CMainWindow:
class CMainWindow : public CFrameWnd
{
public:
	CMainWindow();
	~CMainWindow();

	CBitmap		m_ballBmp;
	CRect		m_winRect;		// Game Board
	CRect		m_scoresRect;	// Scores Area
	
	CString	m_appName;
	CString	m_currGame;
	
	// Status
	BOOL	m_paused;		// Pause On/Off Switch
	BOOL	m_started;		// Started Game Flag        
	BOOL	m_running;		// Ball Running Flag
	BOOL	m_areScoresShown;
	
	// Flags
	BOOL	m_sound;
	BOOL	m_effect;
	BOOL	m_hiSpeed;
	BOOL	m_largeBats;
	BOOL	m_accelerated;                      
	BOOL	m_isFirstPlayerTurn;	// Pelota Game Flag
	BOOL	m_scoresAlwaysOnTop;
	
	// Game Speeds
	int m_computerSpeed;
	int m_hiSpeedBallFactor;
	int m_lowSpeedBallFactor;
	
	// Bats Extents
	int m_batHeight;
	int m_bigBatHeight;
	int m_smallBatHeight;
	
	// GameBoard Colors
	COLORREF	m_ink;
	COLORREF	m_paper;
	
	// Ball Data          
	int m_ballWidth;
	int	m_ballXPos;
	int	m_ballYPos;
	int m_ballXPrevPos;
	int m_ballYPrevPos;
	int	m_ballXVel;
	int	m_ballYVel;
	int m_ballEffect;		// 0=Min, 100=Max  -=ClkWs,+=AntiClkWs
	int m_ballNextEffect;	//
	
	// Player One Data
	UINT	m_upKeyP1;
	UINT	m_downKeyP1;
		
	int	m_velP1;                    
	int	m_posP1;
    int	m_prevPosP1;
	int m_firstBatXPosP1;
	int m_secondBatXPosP1;
    
	int	m_scoreP1;
	
	// Player Two Data
	UINT	m_upKeyP2;
	UINT	m_downKeyP2;
		
	int	m_velP2;
	int	m_posP2;
    int	m_prevPosP2;
	int m_firstBatXPosP2;
	int m_secondBatXPosP2;

	int	m_scoreP2;
	
	// Miscellaneous            
	int m_effectAngle;
	int m_winnerScore;
	int m_computerCount;

	//{{AFX_MSG(CMainWindow)
	afx_msg void OnGameNew();
	afx_msg void OnGamePause();
	afx_msg void OnGamePelota();
	afx_msg void OnGameTennis();
	afx_msg void OnGameHockey();
	afx_msg void OnGameTraining();
	afx_msg void OnGameColors();
	afx_msg void OnGamePlayersHuman();
	afx_msg void OnGamePlayersComputer();
	afx_msg void OnGameKeyboard();
	afx_msg void OnGameScoresalwaysontop();
	afx_msg void OnGameSound();
	afx_msg void OnGameHighspeed();
	afx_msg void OnGameBalleffect();
	afx_msg void OnGameAcceleration();
	afx_msg void OnGameLargerackets();
	afx_msg void OnGameAdvancedoptions();
	afx_msg void OnUpdateGamePause(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGamePlayersHuman(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGamePlayersComputer(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameScoresalwaysontop(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameSound(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameHighspeed(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameBalleffect(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameAcceleration(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameLargerackets(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGamePelota(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameTennis(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameHockey(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGameTraining(CCmdUI* pCmdUI);
	afx_msg void OnHelpOptions();
	afx_msg void OnHelpAbout();
	afx_msg void OnPaint();
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnMenuSelect(UINT nItemID, UINT nFlags, HMENU hSysMenu);
	afx_msg void OnMove(int x, int y);
	afx_msg void OnWindowPosChanging(WINDOWPOS FAR* lpwndpos);
	afx_msg void OnNcLButtonDblClk(UINT nHitTest, CPoint point);
	afx_msg void OnNcLButtonDown(UINT nHitTest, CPoint point);
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	//}}AFX_MSG

	void InitMainWindow();
	
	void MoveComputerBat();
	void SetRandomBallPos();
	void CalculateBallPos();

	void RedrawBall(BOOL bHideBall = FALSE);
	void RedrawRackets(BOOL bForceDrawing = FALSE);

	void ShowScores();
	void HideScores();
	void DrawScoreDigit(int digit, int xCoor, int yCoor);
	
	// Ball Events
	void BallOut();
	void TopBounce(int nPos);
	void LeftBounce(int nPos);
	void RightBounce(int nPos);
	void BottomBounce(int nPos);
	void CalculateEffect();
	
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

// CTheApp:
// See hello.cpp for the code to the InitInstance member function.
//
class CTheApp : public CWinApp
{
public:
	BOOL InitInstance();
};

/////////////////////////////////////////////////////////////////////////////

#endif // __TVGAMES_H__

